@@ -56,8 +56,10 @@ class AppleUSBAudioDevice;
 #define	kChannelDepth_MONO				1
 #define	kChannelDepth_STEREO			2
 
-#define kMinimumFrameOffset				1
-#define kWallTimeExtraPrecision         10000
+#define kFixedPoint10_14ByteSize		3
+#define	kFixedPoint16_16ByteSize		4
+
+#define kMinimumFrameOffset				5
 #define kRefreshInterval				128
 #define kRefreshCount					8
 
@@ -73,11 +75,13 @@ class IOSyncer;
 class AppleUSBAudioEngine;
 class AppleUSBAudioPlugin;
 
+/* THIS LOOKS UNUSED
 typedef struct FrameListWriteInfo {
     AppleUSBAudioEngine *				audioEngine;
     UInt32								frameListNum;
     UInt32								retryCount;
 } FrameListWriteInfo;
+*/
 
 class AppleUSBAudioEngine : public IOAudioEngine {
     friend class AppleUSBAudioDevice;
@@ -100,9 +104,12 @@ public:
 	static void requestiSubClose (IOAudioEngine * audioEngine);
 
 	static void sampleRateHandler (void * target, void * parameter, IOReturn result, IOUSBIsocFrame * pFrames);
-
+	#if PRIMEISOCINPUT
+	static void primeInputPipeHandler (void * object, void * parameter, IOReturn result, IOUSBLowLatencyIsocFrame * pFrames);
+	#endif
     static void readHandler (void * object, void * frameListIndex, IOReturn result, IOUSBLowLatencyIsocFrame * pFrames);
     static void writeHandler (void * object, void * parameter, IOReturn result, IOUSBLowLatencyIsocFrame * pFrames);
+	static void writeHandlerForUHCI (void * object, void * parameter, IOReturn result, IOUSBLowLatencyIsocFrame * pFrames);
 	static IOReturn iSubAttachChangeHandler (IOService *target, IOAudioControl *passThruControl, SInt32 oldValue, SInt32 newValue);
 	virtual IOReturn iSubAttachChangeCall (Boolean isiSubCompatible);
 	virtual IOReturn pluginDeviceRequest (IOUSBDevRequest * request, IOUSBCompletion * completion);
@@ -111,111 +118,122 @@ public:
 	static void	pluginLoaded (AppleUSBAudioEngine * usbAudioEngineObject);
 
 protected:
-	volatile UInt32						currentFrameList;
-	volatile UInt32						shouldStop;
-	UInt64								usbFrameToQueueAt;
-	UInt64								nextSynchReadFrame;
+	volatile UInt32						mCurrentFrameList;
+	volatile UInt32						mShouldStop;
+	UInt64								mUSBFrameToQueue;
+	UInt64								mNextSyncReadFrame;
 	
 	UInt64 *							mFrameQueuedForList;
-	UInt64								mWallTimePerUSBCycle;
-	UInt64								mReferenceUSBFrame;
-	AbsoluteTime						mReferenceWallTime;
-	UInt64								mNewReferenceUSBFrame;
-	AbsoluteTime						mNewReferenceWallTime;
-	UInt64								mLastUSBFrame;
-	UInt64								mLastWallTime_nanos;
 	#if DEBUGTIMESTAMPS
 		UInt64								mLastTimeStamp_nanos;
 		UInt64								mLastStampDifference;
 		SInt64								mStampDrift;
 		UInt64								mLastWrapFrame;
 	#endif
-	UInt8								mAnchorResetCount;
-	UInt8								mHubSpeed;
-	IOTimerEventSource *				mUpdateTimer;
+	// UInt64							mReferenceUSBFrame;
+	// AbsoluteTime						mReferenceWallTime;
+	bool								mSplitTransactions;
+	IOLock *							mCoalescenceMutex;
 	
-	IOUSBLowLatencyIsocFrame *			theUSBIsocFrames;
-	IOUSBIsocFrame						theSampleRateFrame;
-	IOUSBLowLatencyIsocCompletion *		usbCompletion;
-	IOUSBIsocCompletion					sampleRateCompletion;
-	IOUSBInterface *					streamInterface;
-	IOUSBPipe *							thePipe;
-	IOUSBPipe *							theAssociatedPipe;
-	IOSyncer *							mSyncer;
-	AppleUSBAudioDevice *				usbAudioDevice;
+	IOUSBLowLatencyIsocFrame *			mUSBIsocFrames;
+	IOUSBIsocFrame						mSampleRateFrame;
+	IOUSBLowLatencyIsocCompletion *		mUSBCompletion;
+	IOUSBIsocCompletion					mSampleRateCompletion;
+	IOUSBInterface *					mStreamInterface;
+	IOUSBPipe *							mPipe;
+	IOUSBPipe *							mAssociatedPipe;
+	IOSyncer *							mDeviceAndEngineSyncer;
+	AppleUSBAudioDevice *				mUSBAudioDevice;
 	IOBufferMemoryDescriptor *			mUSBBufferDescriptor;
-	IOBufferMemoryDescriptor *			sampleBufferMemoryDescriptor;
-	IOMultiMemoryDescriptor *			theWrapRangeDescriptor;
-	IOSubMemoryDescriptor *				theWrapDescriptors[2];
-	IOSubMemoryDescriptor **			soundBufferDescriptors;
-	IOMemoryDescriptor *				neededSampleRateDescriptor;
-//	IOVirtualRange						theWrapRanges[2];
-	IONotifier *						streamNotifier;
-	void *								readBuffer;
-	void *								theStartBuffer;
-	UInt32 *							aveSampleRateBuf;		// only needs to be 3 bytes for the 10.14 value
-	IOTimerEventSource *				startTimer;
-	thread_call_t						iSubTeardownThreadCall;
+	IOBufferMemoryDescriptor *			mSampleBufferMemoryDescriptor;
+	#if PRIMEISOCINPUT
+	IOUSBLowLatencyIsocFrame *			mPrimeInputIsocFrames;
+	IOUSBLowLatencyIsocCompletion		mPrimeInputCompletion;
+	#endif
+	IOMultiMemoryDescriptor *			mWrapRangeDescriptor;
+	IOSubMemoryDescriptor *				mWrapDescriptors[2];
+	IOSubMemoryDescriptor **			mSampleBufferDescriptors;
+	IOMemoryDescriptor *				mAssociatedEndpointMemoryDescriptor;
+	
+	// UHCI additions
+	
+	UInt32								mSampleBufferSizeExtended;			
+	UInt16								mNumFramesInFirstList;
+	IOUSBLowLatencyIsocCompletion		mExtraUSBCompletion;
+	UInt16								mAverageFrameSize;					// These sizes are in bytes.
+	UInt16								mAlternateFrameSize;
+	UInt16								mReadUSBFrameSize;
+
+	// end UHCI additions
+	
+	void *								mReadBuffer;			
+	UInt32 *							mAverageSampleRateBuffer;		// needs to be 4 bytes for a 10.14 or 16.16 value
+	thread_call_t						miSubTeardownThreadCall;
 	thread_call_t						mPluginInitThread;
-	thread_call_t						iSubOpenThreadCall;
-//	thread_call_t						firstUSBFrameCompletionThread;
-	IOAudioSampleRate					curSampleRate;
-	UInt32								previouslyPreparedBufferOffset;
-	UInt32								safeToEraseTo;
-	UInt32								lastSafeErasePoint;
-	UInt32								theStartBufferSize;
-	UInt32								previousClippedToFrame;
-	UInt32								readUSBFrameListSize;
-	UInt32								bufferOffset;
-	UInt32								averageSampleRate;
-	UInt32								sum;					// This is the number of samples we have sent in the previous frames in this period
-	UInt32								numUSBFrameLists;
-	UInt32								numUSBFramesPerList;
-	UInt32								numUSBFrameListsToQueue;
-	UInt32								bufferSize;
-	UInt32								bytesPerSampleFrame;
-	UInt32								safetyOffset;
-	UInt32								fractionalSamplesRemaining;
-	UInt16								samplesThisFrame;
-	UInt8								ourInterfaceNumber;
-	UInt8								alternateSettingID;
-	UInt8								refreshInterval;
-	UInt8								framesUntilRefresh;
-	UInt8								direction;
-	Boolean								inCompletion;
-	Boolean								usbStreamRunning;
-	Boolean								notifyDeviceOfStop;
-	Boolean								startingEngine;
-	Boolean								startiSub;
-	Boolean								justResetClipPosition;		// aml 3.29.02 added to match AOA
-	Boolean								gotRequestClose;
-	Boolean								gotStopCall;
-	Boolean								iSubIsOpen;
-	Boolean								terminatingDriver;
-	IOAudioStream *						mainStream;
+	thread_call_t						miSubOpenThreadCall;
+	IOAudioSampleRate					mCurSampleRate;
+	UInt32								mLastPreparedBufferOffset;
+	UInt32								mSafeErasePoint;
+	UInt32								mLastSafeErasePoint;
+	UInt32								mLastClippedFrame;
+	UInt32								mReadUSBFrameListSize;
+	UInt32								mBufferOffset;
+	UInt32								mAverageSampleRate;
+	UInt32								mNumUSBFrameLists;
+	UInt32								mNumUSBFramesPerList;
+	UInt32								mNumUSBFrameListsToQueue;
+	UInt32								mSampleBufferSize;
+	UInt32								mBytesPerSampleFrame;
+	UInt32								mFractionalSamplesLeft;
+	#if DEBUGLATENCY
+		UInt32								mLastFrameListSize;
+		UInt32								mThisFrameListSize;
+	#endif
+	UInt16								mSampleSize;
+	UInt8								mInterfaceNumber;
+	UInt8								mAlternateSettingID;
+	UInt8								mRefreshInterval;
+	UInt8								mFramesUntilRefresh;
+	UInt8								mFeedbackPacketSize;
+	UInt8								mDirection;
+	Boolean								mInCompletion;
+	Boolean								mUSBStreamRunning;
+	Boolean								mStartiSub;
+	Boolean								mJustResetClipPosition;	// aml 3.29.02 added to match AOA
+	Boolean								miSubIsOpen;
+	Boolean								mTerminatingDriver;
+	Boolean								mUHCISupport;
+	IOAudioStream *						mMainStream;
 
 	IONotifier *						mPluginNotification;
 	AppleUSBAudioPlugin *				mPlugin;
 
 	// For iSub
-	IOMemoryDescriptor *				iSubBufferMemory;
-	UInt32								iSubLoopCount;
-	SInt32								iSubBufferOffset;		// Where we are writing to in the iSub buffer
-	IOService *							ourProvider;
-	IONotifier *						iSubEngineNotifier;
-	IOAudioToggleControl *				iSubAttach;
-	AppleiSubEngine *					iSubEngine;
-	AppleiSubEngine *					oldiSubEngine;
-	float *								lowFreqSamples;
-	float *								highFreqSamples;
-	PreviousValues						filterState;
-	PreviousValues						filterState2;
-	PreviousValues						phaseCompState;
-	float								srcPhase;
-	float								srcState;
-	Boolean								needToSync;
-	Boolean								needTimeStamps;
-
+	IOMemoryDescriptor *				miSubBufferMemory;
+	UInt32								miSubLoopCount;
+	SInt32								miSubBufferOffset;		// Where we are writing to in the iSub buffer
+	IONotifier *						miSubEngineNotifier;
+	IOAudioToggleControl *				miSubAttachToggle;
+	AppleiSubEngine *					miSubEngine;
+	AppleiSubEngine *					mOldiSubEngine;
+	float *								mLowFreqSamples;
+	float *								mHighFreqSamples;
+	PreviousValues						mFilterState;
+	PreviousValues						mFilterState2;
+	PreviousValues						mPhaseCompState;
+	float								mSrcPhase;
+	float								mSrcState;
+	Boolean								mNeedToSync;
+	Boolean								mNeedTimeStamps;
+	#if DEBUGLATENCY
+		bool								mHaveClipped;
+	#endif
+	bool								mHaveTakenFirstTimeStamp;
+	
+	// Default stream format and sample rate are stored.
+	IOAudioStreamFormat					mDefaultAudioStreamFormat;
+	IOAudioSampleRate					mDefaultAudioSampleRate;
+	
 	static inline IOFixed IOUFixedDivide(UInt32 a, UInt32 b)
 	{
 		return (IOFixed)((((UInt64) a) << 16) / ((UInt64) b));
@@ -236,7 +254,6 @@ protected:
 
     static bool audioDevicePublished (AppleUSBAudioEngine *audioEngine, void *ref, IOService *newService);
     static bool	iSubEnginePublished (AppleUSBAudioEngine * usbAudioEngineObject, void * refCon, IOService * newService);
-	static void waitForFirstUSBFrameCompletion (OSObject * owner, IOTimerEventSource * sender);
 
 	virtual bool willTerminate (IOService * provider, IOOptionBits options);
 
@@ -265,15 +282,20 @@ protected:
 	virtual IOReturn convertInputSamples (const void *sampleBuf, void *destBuf, UInt32 firstSampleFrame, UInt32 numSampleFrames, const IOAudioStreamFormat *streamFormat, IOAudioStream *audioStream);
 	
     virtual IOReturn performFormatChange (IOAudioStream *audioStream, const IOAudioStreamFormat *newFormat, const IOAudioSampleRate *newSampleRate);
+	virtual	IOReturn controlledFormatChange (IOAudioStream *audioStream, const IOAudioStreamFormat *newFormat, const IOAudioSampleRate *newSampleRate);
 	void CalculateSamplesPerFrame (UInt32 sampleRate, UInt16 * averageFrameSize, UInt16 * additionalSampleFrameFreq);
+	#if DEBUGLATENCY
+	virtual UInt64 getQueuedFrameForSample (UInt32 sampleFrame);
+	#endif
+	virtual	void takeTimeStamp (bool incrementLoopCount = true, AbsoluteTime *timestamp = NULL);
+	#if PRIMEISOCINPUT
+	virtual	void primeInputPipe (IOUSBPipe * pipeToPrime, UInt32 bytesPerUSBFrame);
+	#endif
+	virtual AbsoluteTime generateTimeStamp (UInt32 usbFrameIndex, UInt32 preWrapBytes, UInt32 byteCount);
+	virtual IOReturn copyAnchor (UInt64 * anchorFrame, AbsoluteTime * anchorTime); 
 	
-	IOReturn getAnchorFrameAndTimeStamp (UInt64 *frame, AbsoluteTime *time);
-	IOReturn getFrameAndTimeStamp (UInt64 *frame, AbsoluteTime *time);
-	UInt64 jitterFilter (UInt64 invCoefficient, UInt64 prev, UInt64 curr);
-	void updateWallTimePerUSBCycle ();
-	AbsoluteTime generateTimeStamp (UInt32 usbFrameIndex, UInt32 preWrapBytes, UInt32 byteCount);
-	static	void TimerAction (OSObject * owner, IOTimerEventSource * sender);
-	void doTimerAction (IOTimerEventSource * timer);
+	// for UHCI
+	virtual IOReturn eraseOutputSamples(const void *mixBuf, void *sampleBuf, UInt32 firstSampleFrame, UInt32 numSampleFrames, const IOAudioStreamFormat *streamFormat, IOAudioStream *audioStream);
 
 };
 
