@@ -149,24 +149,34 @@ SInt32 *AppleUSBTrinityAudioDevice::getProperEQSettings () {
     powerProperty = parent->getProperty ("AAPL,current-available");
     FailIf (!powerProperty, Exit);
 
-    if (powerNum = OSDynamicCast (OSNumber, powerProperty)) {
+    if (powerNum = OSDynamicCast (OSNumber, powerProperty)) 
+	{
         availablePower = powerNum->unsigned32BitValue ();
-    } else if (powerData = OSDynamicCast (OSData, powerProperty)) {
+    } 
+	else if (powerData = OSDynamicCast (OSData, powerProperty)) 
+	{
         availablePower = *(UInt32 *)powerData->getBytesNoCopy ();
     }
 
     debugIOLog ("AppleUSBTrinityAudioDevice[0x%x]: Hub (%d) found.", this, availablePower);
 
-    if (availablePower >= 4000) {
+    if (availablePower >= 4000) 
+	{
 		debugIOLog ("Usinging 4 amp EQ");
         settings = power4AEQSettings;
-    } else if (availablePower >= 3000) {
+    } 
+	else if (availablePower >= 3000) 
+	{
 		debugIOLog ("Usinging 3 amp EQ");
         settings = power3AEQSettings;
-	} else if (availablePower >= 1500) {
+	} 
+	else if (availablePower >= 1500) 
+	{
 		debugIOLog ("Usinging 1.5 amp EQ");
 		settings = power1500mAEQSettings;
-    } else if (availablePower >= 500) {
+    }
+	else if (availablePower >= 500) 
+	{
 		debugIOLog ("Usinging 0.5 amp EQ");
         settings = power500mAEQSettings;
     }
@@ -188,9 +198,11 @@ IOReturn AppleUSBTrinityAudioDevice::downloadEQ (SInt32 * eqSettings) {
     result = kIOReturnBadArgument;
     FailWithAction (!eqSettings, debugIOLog("AppleUSBTrinityAudioDevice::downloadEQ() - error no EQ settings available.\n"), Exit);
 
-    for (eqIndex = 0, xdfpAddr = XDFP_STARTING_EQ_ADDR; eqIndex < EQ_TABLE_SIZE; eqIndex++, xdfpAddr++) {
+    for (eqIndex = 0, xdfpAddr = XDFP_STARTING_EQ_ADDR; eqIndex < EQ_TABLE_SIZE; eqIndex++, xdfpAddr++) 
+	{
         result = xdfpWrite (xdfpAddr, eqSettings[eqIndex]);
-        if (result != kIOReturnSuccess) {
+        if (result != kIOReturnSuccess) 
+		{
             debugIOLog ("AppleUSBTrinityAudioDevice::downloadEQ () - error writing settings %d: 0x%x", eqIndex, result);
             break;
         }
@@ -240,9 +252,10 @@ IOReturn AppleUSBTrinityAudioDevice::performPowerStateChange (IOAudioDevicePower
 
 	result = super::performPowerStateChange (oldPowerState, newPowerState, microSecsUntilComplete);
 
-	if (oldPowerState == kIOAudioDeviceSleep) {
+	if (oldPowerState == kIOAudioDeviceSleep) 
+	{
 		debugIOLog ("G4 Cube speakers: Waking from sleep - downloading power settings to speakers.");
-		eqRetryCount = 0;
+		mEQRetryCount = 0;
 		attemptEQDownload ();
 	}
 
@@ -270,13 +283,17 @@ Exit:
 }
 
 void AppleUSBTrinityAudioDevice::attemptEQDownload (void) {
-    debugIOLog ("AppleUSBTrinityAudioDevice::attemptEQDownload () - eqRetryCount = %d", eqRetryCount);
+    debugIOLog ("AppleUSBTrinityAudioDevice::attemptEQDownload () - mEQRetryCount = %d", mEQRetryCount);
 
 	FailIf (NULL == mControlInterface, Exit);
-	if (downloadEQ () != kIOReturnSuccess) {
-		if (++eqRetryCount < MAX_EQ_DOWNLOAD_RETRIES) {
+	if (downloadEQ () != kIOReturnSuccess) 
+	{
+		if (++mEQRetryCount < MAX_EQ_DOWNLOAD_RETRIES) 
+		{
 			scheduleEQDownloadRetry ();
-		} else {
+		} 
+		else 
+		{
 			debugIOLog ("G4 Cube Speakers: too many retries of downloadEQ () - aborting.");
 		}
 	}
@@ -287,7 +304,8 @@ Exit:
 
 void AppleUSBTrinityAudioDevice::retryEQDownload (void * arg) {
     AppleUSBTrinityAudioDevice * device = (AppleUSBTrinityAudioDevice *)arg;
-    if (device) {
+    if (device) 
+	{
         device->attemptEQDownload ();
 		device->release ();	// it's now safe for our driver to be unloaded if needed.
     }
